#include "m_i2c.h"


void i2c_init ()
{
	TWSR = 0;
	TWBR = 32;
}

/*******************************************************************************************************************/
uint8_t i2c_write (uint8_t data)
{
	// Locals
	uint8_t twst;

#if (DEBUG == 1)
	uart_puts ("TWI write [");
	uart_putc (data);
	uart_puts ("]:");
#endif

	// Set TWI registers and start transmission of data
	TWDR = data;
	TWCR = (1<<TWINT) | (1<<TWEN);
	
	// wail until transmission completed and ACK/NACK has been received
	while(!(TWCR & (1<<TWINT)));
	
	// check value of TWI Status Register. Mask prescaler bits.
	twst = TW_STATUS & 0xF8;

#if (DEBUG == 1)
	switch (twst)
	{
		case TW_MT_SLA_ACK:
			uart_puts ("TW_MT_SLA_ACK\n");
		break;

		case TW_MT_SLA_NACK:
			uart_puts ("TW_MT_SLA_NACK\n");
		break;

		case TW_MT_DATA_ACK:
			uart_puts ("TW_MT_DATA_ACK\n");
		break;

		case TW_MT_DATA_NACK:
			uart_puts ("TW_MT_DATA_NACK\n");
		break;

		case TW_MT_ARB_LOST:
			uart_puts ("TW_MT_ARB_LOST\n");
		break;

		case TW_MR_SLA_ACK:
			uart_puts ("TW_MR_SLA_ACK\n");
		break;

		case TW_MR_SLA_NACK:
			uart_puts ("TW_MR_SLA_NACK\n");
		break;

		case TW_MR_DATA_NACK:
			uart_puts ("TW_MR_DATA_NACK\n");
		break;

		case TW_ST_SLA_ACK:
			uart_puts ("TW_ST_SLA_ACK\n");
		break;

		case TW_ST_ARB_LOST_SLA_ACK:
			uart_puts ("TW_ST_ARB_LOST_SLA_ACK\n");
		break;

		case TW_ST_DATA_ACK:
			uart_puts ("TW_ST_DATA_ACK\n");
		break;

		case TW_ST_DATA_NACK:
			uart_puts ("TW_ST_DATA_NACK\n");
		break;

		case TW_ST_LAST_DATA:
			uart_puts ("TW_ST_LAST_DATA\n");
		break;

		default:
			uart_putc (twst);
			uart_putc('\n');
		break;
	}
#endif

	return twst;
}

/*******************************************************************************************************************/
uint8_t i2c_start ()
{
	// Locals
	uint8_t	 twst;

#if (DEBUG == 1)
	uart_puts ("TWI start: ");
#endif
	
	// send START condition
	TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);

	// wait until transmission completed
	while(!(TWCR & (1<<TWINT)));

	// check value of TWI Status Register. Mask prescaler bits.
	twst = TW_STATUS & 0xF8;

#if (DEBUG == 1)
	switch (twst)
	{
		case TW_START:
			uart_puts ("TW_START\n");
		break;

		case TW_REP_START:
			uart_puts ("TW_REP_START\n");
		break;

		default:
			uart_putc (twst);
			uart_putc('\n');
		break;
	}
#endif
	
	return twst;
}

/*******************************************************************************************************************/
uint8_t i2c_readNak ()
{
#if (DEBUG == 1)
	uart_puts ("TWI readNAK [");
#endif

	// Set TWI registers and start reading data
	TWCR = (1<<TWINT) | (1<<TWEN);
	// wait until reading completed
	while(!(TWCR & (1<<TWINT)));

#if (DEBUG == 1)
	uart_putc (TWDR);
	uart_puts ("]\n");
#endif

    return TWDR;
}

/*******************************************************************************************************************/
uint8_t i2c_readAck ()
{
#if (DEBUG == 1)
	uart_puts ("TWI readACK [");
#endif

	// Set TWI registers and start reading data
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWEA);
	// wait until reading completed
	while(!(TWCR & (1<<TWINT)));

#if (DEBUG == 1)
	uart_putc (TWDR);
	uart_puts ("]\n");
#endif

    return TWDR;
}

/*******************************************************************************************************************/
void i2c_stop ()
{
#if (DEBUG == 1)
	uart_puts ("TWI stopped\n");
#endif

	TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWSTO);
}











